@@ -51,17 +51,6 @@ class RackspaceService(object):
         driver = cls(fedimg.RACKSPACE_USER, fedimg.RACKSPACE_API_KEY,
                      region=self.regions[0])
 
-        # create image from offical Fedora image on Rackspace
-
-        # deploy node
-        name = 'fedimg AMI builder'
-        # TODO: Make automatically-created /dev/sda be deleted on termination
-        mappings = [{'VirtualName': None,
-                     'Ebs': {'VolumeSize': 12,  # 12 GB should be enough
-                             'VolumeType': 'standard',
-                             'DeleteOnTermination': 'true'},
-                     'DeviceName': '/dev/sdb'}]
-
-        # register that volume as an image
+        # create image from official Fedora image on Rackspace
 
         # emit a fedmsg, etc
