@@ -8,7 +8,7 @@
 #
 # fedimg is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 # Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public
@@ -23,17 +23,25 @@ import logging
 log = logging.getLogger("fedmsg")
 
 from fedimg.services.ec2 import EC2Service
+from fedimg.util import virt_types_from_url
 
 
-def upload(urls):
-    """ Takes a list of one or more .raw.xz image files and
-    sends them off to cloud services for registration. """
+def upload(pool, urls, compose_meta):
+    """ Takes a list (urls) of one or more .raw.xz image files and
+    sends them off to cloud services for registration. The upload
+    jobs threadpool must be passed as `pool`."""
 
     log.info('Starting upload process')
 
+    services = []
+
     for url in urls:
         # EC2 upload
-        ec2 = EC2Service()
-        ec2.upload(url)
+        log.info("  Preparing to upload %r" % url)
+        for vt in virt_types_from_url(url):
+            services.append(EC2Service(url, virt_type=vt,
+                                       vol_type='standard'))
+            services.append(EC2Service(url, virt_type=vt,
+                                       vol_type='gp2'))
 
-    log.info('Upload process finished')
+    results = pool.map(lambda s: s.upload(compose_meta), services)
