@@ -1,5 +1,5 @@
 # This file is part of fedimg.
-# Copyright (C) 2014 Red Hat, Inc.
+# Copyright (C) 2014-2015 Red Hat, Inc.
 #
 # fedimg is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -17,17 +17,20 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 # Authors:  David Gay <dgay@redhat.com>
+#           Ralph Bean <rbean@redhat.com>
 #
 
 """
 Utility functions for fedimg.
 """
 
+import functools
 import socket
 import subprocess
 
 import paramiko
 from libcloud.compute.types import Provider
+from libcloud.compute.providers import get_driver
 
 import fedimg
 
@@ -44,46 +47,34 @@ def get_file_arch(file_name):
         return None
 
 
-def get_rawxz_url(task_result):
-    """ Returns the URL of the raw.xz file produced by the Koji task whose
-    output files are passed as a list via the task_result argument. """
-    # I think there might only ever be one qcow2 file per task,
-    # but doing it this way plays it safe.
-    file_name = [f for f in task_result['files'] if f.endswith('.raw.xz')][0]
-    task_id = task_result['task_id']
+def get_rawxz_urls(location, images):
+    """ Iterates through all the images metadata and returns the url of .raw.xz
+    files.
+    """
+    rawxz_list = [f['path'] for f in images if f['path'].endswith('.raw.xz')]
+    if not rawxz_list:
+        return []
 
-    # extension to base URL to exact file directory
-    koji_url_extension = "/{}/{}".format(str(task_id)[3:], str(task_id))
-    full_file_location = fedimg.BASE_KOJI_TASK_URL + koji_url_extension
-
-    return full_file_location + "/{}".format(file_name)
+    return map((lambda path: '{}/{}'.format(location, path)), rawxz_list)
 
 
-def get_virt_type(file_name):
-    """ Takes a file name (probably of a .raw.xz image file) and returns
-    the suspected virtualization type that the image file should be
-    registered as. """
-    file_name = file_name.lower()
+def virt_types_from_url(url):
+    """ Takes a URL to a .raw.xz image file) and returns the suspected
+        virtualization type that the image file should be registered as. """
+    file_name = url.split('/')[-1].lower()
     if file_name.find('atomic') != -1:
         # hvm is required for atomic images
-        return 'hvm'
+        return ['hvm']
     else:
-        # otherwise, use the default (paravirtual)
-        return 'paravirtual'
+        # otherwise, build the AMIs with both virtualization types
+        return ['hvm', 'paravirtual']
 
 
-def region_to_provider(region):
+def region_to_driver(region):
     """ Takes a region name (ex. 'eu-west-1') and returns
     the appropriate libcloud provider value. """
-    providers = {'ap-northeast-1': Provider.EC2_AP_NORTHEAST,
-                 'ap-southeast-1': Provider.EC2_AP_SOUTHEAST,
-                 'ap-southeast-2': Provider.EC2_AP_SOUTHEAST2,
-                 'eu-west-1': Provider.EC2_EU_WEST,
-                 'sa-east-1': Provider.EC2_SA_EAST,
-                 'us-east-1': Provider.EC2_US_EAST,
-                 'us-west-1': Provider.EC2_US_WEST,
-                 'us-west-2': Provider.EC2_US_WEST_OREGON}
-    return providers[region]
+    cls = get_driver(Provider.EC2)
+    return functools.partial(cls, region=region)
 
 
 def ssh_connection_works(username, ip, keypath):
@@ -101,3 +92,12 @@ def ssh_connection_works(username, ip, keypath):
         pass
     ssh.close()
     return works
+
+
+def safeget(dct, *keys):
+    for key in keys:
+        try:
+            dct = dct[key]
+        except KeyError:
+            return None
+    return dct
