@@ -22,14 +22,16 @@
 import socket
 hostname = socket.gethostname()
 
+NUM_BASE_THREADS = 4
+NUM_ATOMIC_THREADS = 2
+NUM_PORTS = 2 * ((NUM_BASE_THREADS + NUM_ATOMIC_THREADS) + 1)
+
 config = dict(
-    kojiconsumer=True,
+    fedimgconsumer=True,
     endpoints={
         "fedimg.%s" % hostname: [
-            "tcp://127.0.0.1:6009",
-            "tcp://127.0.0.1:6008",
-            "tcp://127.0.0.1:6007",
-            "tcp://127.0.0.1:6006",
+            "tcp://127.0.0.1:60%0.2i" % (i)
+            for i in range(NUM_PORTS)
         ],
     },
 )
